# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################
"""
"""
from glob import glob
from os import system
from os.path import basename
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.types import Text, Integer
from sqlalchemy import Column
from sqlalchemy import create_engine, orm
from pyeole.service import manage_services


CONFIGS = ['/etc/ldap/replication/replication-*.conf', '/etc/ldap/replication/zephir/replication-*.conf']
STORAGE_FILE = '/usr/share/ead2/backend/tmp/replication.db'


Item = declarative_base()


def init_store():
    """
    Initialisation de la base
    """
    engine = create_engine(f'sqlite:///{STORAGE_FILE}' )
    Session = orm.sessionmaker()
    Session.configure(bind=engine)
    session = Session()
    Item.metadata.create_all(engine)
    return session


class Etablissement(Item):
    """
    Définition de la base SQLite pour l'attribution des RID de réplication
    """
    __tablename__ = 'etablissement'
    id = Column("id", Integer, primary_key=True)
    rid = Column("rid", Integer, nullable=False)
    rne = Column("rne", Text, nullable=False)


def gen_replication_config(configfile='/etc/ldap/replication.conf', restart=True):
    """
    Génération du fichier de configuration des fournisseurs
    """
    etabs = {}
    storage = init_store()
    for etab in storage.query(Etablissement).all():
        etabs[etab.rne] = etab.rid
    if etabs != {}:
        index = max(etabs.values())+1
    else:
        index = 0

    result = []
    configs = []
    for glob_conf in CONFIGS:
        configs.extend(glob(glob_conf))
    for config in configs:
        rne = basename(config)[12:-5]
        if rne in etabs:
            rid = etabs[rne]
        else:
            rid = index
            storage.add(Etablissement(rid=index, rne=rne))
            storage.commit()
            index += 1
        try:
            result.append(open(config).read() % rid)
        except:
            import traceback
            traceback.print_exc()
            print(f"fichier de configuration {config} invalide")

    fic = open(configfile, 'w')
    fic.writelines(result)
    fic.close()

    if restart:
        manage_services('restart', 'slapd')
        # relance du service EoleSSO (maj des infos établisements) #12822
        manage_services('reload', 'eole-sso')
